####################################################################################
#
#		:::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]::: 
#      ____                __                                                      
#     / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __   
#    / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // /   
#   /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/    
#        /___/ /___/                 /_/    /_/           /___/ /_/                
#
#
#####################################################################################
#
# newyear.tcl 1.2.1
#
# Description:
#	Displays remaining time until new year.
#
# Installation:
#	1. Make sure you correctly installed egglib_pub.
#	2. Copy deathday.tcl to scripts/ directory.
#	3. !!! AFTER !!! 'source scripts/egglib_pub.tcl' line in your eggdrop.conf add next:
#		source scripts/newyear.tcl
#	4. .rehash the bot
#
# Channel flags:
#	.chanset #chan -nonewyear 	- disable usage of !newyear for #chan
#	.chanset #chan +nonewyear 	- enable usage of !newyear for #chan
#
# Authors: Shrike <shrike@eggdrop.org.ru>
#
# Official support: irc.eggdrop.org.ru @ #eggdrop
# 
#####################################################################################
#
# Version History:
#	
#  		SHR - Shrike
#	
#		Who		DATE	Changes
#   	---     ------  -------------------------------------------------------------
# v1.2.1			
#   	SHR		041230	Fixed displaying "since xxx year..."
# v1.2			
#   	SHR		041230	Added +|- gmt
#						Fixed calculation time.
#						Added bind for new year banner.
# v1.1			
#   	SHR		041125	Fixed a little bug with color ;)
#
# v1.0			
#   	SHR		041123	First public version.
#								
#####################################################################################
setudef flag nopubnewyear

if { ![info exists egglib(ver)] } {
	putlog "***********************************************"
	putlog "             egglib_pub NOT FOUND !"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"	
	return
}

if { [expr {$egglib(ver) < 1.4}] } {
	putlog "***********************************************"
	putlog " YOUR VERSION OF egglib_pub IS TOO OLD !"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"
	putlog " version installed : $egglib(ver)"
	putlog " version required: 1.4"
	return
}

foreach p [array names newyear *] { catch {unset newyear($p) } }
namespace eval newyear {}

#    (thanks to Twin @ Rusnet)
foreach bind [binds "::newyear::*"] {
    catch {unbind [lindex $bind 0] [lindex $bind 1] [lindex $bind 2] [lindex $bind 4]}
}

set newyear(ver) 1.2.1
set newyear(authors) "Shrike <shrike@eggdrop.org.ru>"

bind pub - !  	 ::newyear::pub
bind msg - !  	 ::newyear::msg
bind pub - !newyear  ::newyear::pub
bind msg - !newyear  ::newyear::msg
bind time - "00 00 01 01 *" ::newyear::banner

#############################################################
# ::newyear::duration is a part of egglib 2.0
#############################################################
set newyear(dur_array) {
	{	{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}}}
	{	{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}}} 
	{	{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}}} 
	{	{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}}} 
	{	{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}}} 
	{	{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}} {{} {} {} {}}
		{{} {} {} {}}}
}

proc ::newyear::duration { time {mode 0} {text ""} } {
	global newyear
	set ret ""; set m $mode; if { $text == "" } { set expire [::duration $time]} { set expire $text }
	for { set s1 0 } { $s1 <= 6 } { incr s1 } {
	
		if { $mode == 3 } { set s2 [expr {6-$s1}] } { set s2 $s1 }
		switch -exact $s2 {
			0 {	set re {([0-9]+?)\ year} }
			1 {	set re {([0-9]+?)\ month} }
			2 {	set re {([0-9]+?)\ week} }
			3 {	set re {([0-9]+?)\ day} }
			4 {	set re {([0-9]+?)\ hour} }
			5 {	set re {([0-9]+?)\ minute} }
			6 {	set re {([0-9]+?)\ second} }
		}
		
		# for year, day
		if { [regexp -nocase -- $re $expire g d] } {
			set k $d
			while {$k>10} { incr k -10 }
			if {($d>10 && $d<21) || $k>4 || $d==0} {
				append ret " $d [lindex [lindex [lindex $newyear(dur_array) $m] $s2] 2]"
			} elseif {$k==1} { 
				if {$d==1} { 
					append ret " $d [lindex [lindex [lindex $newyear(dur_array) $m] $s2] 0]" 
				} else { 
					append ret " $d [lindex [lindex [lindex $newyear(dur_array) $m] $s2] 3]" 
				}
			} elseif {$k<5} {
				append ret " $d [lindex [lindex [lindex $newyear(dur_array) $m] $s2] 1]"
			} else {
				append ret " $d [lindex [lindex [lindex $newyear(dur_array) $m] $s2] 2]"
			}
		}	
		
		if { $m == 3 && $ret != "" } { set m 2 }
	}

	set ret [string trim $ret]
	return $ret
}

proc ::newyear::msg {nick uhost hand args} {
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	::newyear::newyear $nick $uhost $hand $nick $args
}

proc ::newyear::pub {nick uhost hand chan args} {
	if {[channel get $chan nopubnewyear]} return
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	::newyear::newyear $nick $uhost $hand $chan $args
}

proc ::newyear::banner { mi ho da mo ye } {
	foreach chan [channels] {
		if {![channel get $chan nopubnewyear]} {
			::egglib::out "" $chan "\0036*\0032*\00310* \0034\002   $chan !!! \002\00310*\0032*\0036*"
		}
	}
}

proc ::newyear::newyear {nick uhost hand chan args} {
	if { [llength [split [lindex $args 0] ] ] > 1 || [llength [split [lindex $args 0] ] ] == 0 } { set args [split [lindex $args 0]] }
	set gmt [string trim [lindex $args 0]]
	set year [string trim [lindex $args 1]]
	
	if { ![regexp -- {^\-|\+[0-9]+$} $gmt ] } { set year $gmt; set gmt "" }
	
	if { $gmt == "" } {
		set gmt [expr { [clock format [clock scan "today"] -format "%H"] - [clock format [clock scan "today" -gmt 1] -format "%H" -gmt 1]}]
	}
	
	if { [expr {$gmt >12 || $gmt < -12}] } {
		::egglib::outh $nick $chan "!" "\[+|- \] \[\]"
		return
	}
	
	set thisy [clock format [clock scan "today" -gmt 1] -format "%Y" -gmt 1]
	
	if { $year == "" } {
		set year [clock format [clock scan "next year" -gmt 1] -format "%Y" -gmt 1]
	}
	
	if { ![regexp -- {^[0-9]+$} $year ] } {
		::egglib::outh $nick $chan "!" "\[+|- \] \[\]"
		return
	}
	
	if { [expr {$year < 1970}] } { 
		::egglib::out $nick $chan "    ... >8]"
		return	
	}
	
	if { [expr {$year >= 2038}] } {
		::egglib::out $nick $chan "      ... >8]"
		return
	}
	
	
	if { [catch {
		set today [expr {[clock scan "today" -gmt 1]+$gmt*60*60}]
		set next [clock scan "$year-01-01 00:00 GMT" -gmt 1]
	} err] } {
		::egglib::outh $nick $chan "!" "\[+|- \] \[\]"
		return
	}
	
	if { [expr {$next - $today < 0}] } {
		set out [::newyear::duration [expr {$today - $next}] 0]
		if { $year == $thisy } {
			::egglib::out $nick $chan "\0035    - \00314$out !\003"
		} {
			::egglib::out $nick $chan "\0035 $year   -\00314 $out\003 !"
		}
		return
	}
	
	set out [::newyear::duration [expr {$next - $today}]]
	
	if { $thisy+1 == $year } {
		::egglib::out $nick $chan "\0035    -\00314 $out\003 !"
	} {
		::egglib::out $nick $chan "\0035 $year   -\00314 $out\003 !"
	}
}

putlog "newyear.tcl v$newyear(ver) by $newyear(authors) loaded"
